'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = erode;

var _Image = require('../Image');

var _Image2 = _interopRequireDefault(_Image);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Erosion is one of two fundamental operations (with dilatation) in morphological
 * image processing from which all other morphological operations are based (from Wikipedia).
 * Replaces each value with it's local minimum among the pixels with a kernel value of 1.
 * http://docs.opencv.org/2.4/doc/tutorials/imgproc/erosion_dilatation/erosion_dilatation.html
 * https://en.wikipedia.org/wiki/Erosion_(morphology)
 * @memberof Image
 * @instance
 * @param {object} [options]
 * @param {Array<Array<number>>} [options.kernel] - The kernel can only have ones and zeros. Default: [[1, 1, 1], [1, 1, 1], [1, 1, 1]]
 * @param {number} [options.iterations=1] - The number of successive erosions
 * @return {Image}
 */
function erode(options = {}) {
  var _options$kernel = options.kernel,
      kernel = _options$kernel === undefined ? [[1, 1, 1], [1, 1, 1], [1, 1, 1]] : _options$kernel,
      _options$iterations = options.iterations,
      iterations = _options$iterations === undefined ? 1 : _options$iterations;


  this.checkProcessable('erode', {
    bitDepth: [1, 8, 16],
    components: 1,
    alpha: 0
  });
  if (kernel.columns % 2 === 0 || kernel.rows % 2 === 0) {
    throw new TypeError('erode: The number of rows and columns of the kernel must be odd');
  }

  var onlyOnes = true;
  outer: for (var row of kernel) {
    for (var value of row) {
      if (value !== 1) {
        onlyOnes = false;
        break outer;
      }
    }
  }

  var result = this;
  for (var i = 0; i < iterations; i++) {
    if (this.bitDepth === 1) {
      if (onlyOnes) {
        var newImage = result.clone();
        result = erodeOnceBinaryOnlyOnes(result, newImage, kernel.length, kernel[0].length);
      } else {
        var _newImage = _Image2.default.createFrom(result);
        result = erodeOnceBinary(result, _newImage, kernel);
      }
    } else if (onlyOnes) {
      var _newImage2 = _Image2.default.createFrom(result);
      result = erodeOnceGreyOnlyOnes(result, _newImage2, kernel.length, kernel[0].length);
    } else {
      var _newImage3 = _Image2.default.createFrom(result);
      result = erodeOnceGrey(result, _newImage3, kernel);
    }
  }
  return result;
}

function erodeOnceGrey(img, newImage, kernel) {
  var kernelWidth = kernel.length;
  var kernelHeight = kernel[0].length;
  var radiusX = (kernelWidth - 1) / 2;
  var radiusY = (kernelHeight - 1) / 2;
  for (var y = 0; y < img.height; y++) {
    for (var x = 0; x < img.width; x++) {
      var min = img.maxValue;
      for (var jj = 0; jj < kernelHeight; jj++) {
        for (var ii = 0; ii < kernelWidth; ii++) {
          if (kernel[ii][jj] !== 1) continue;
          var i = ii - radiusX + x;
          var j = jj - radiusY + y;
          if (i < 0 || j < 0 || i >= img.width || j >= img.height) continue;
          var value = img.getValueXY(i, j, 0);
          if (value < min) min = value;
        }
      }
      newImage.setValueXY(x, y, 0, min);
    }
  }
  return newImage;
}

function erodeOnceGreyOnlyOnes(img, newImage, kernelWidth, kernelHeight) {
  var radiusX = (kernelWidth - 1) / 2;
  var radiusY = (kernelHeight - 1) / 2;

  var minList = [];
  for (var x = 0; x < img.width; x++) {
    minList.push(0);
  }

  for (var y = 0; y < img.height; y++) {
    for (var _x = 0; _x < img.width; _x++) {
      var min = img.maxValue;
      for (var h = Math.max(0, y - radiusY); h < Math.min(img.height, y + radiusY + 1); h++) {
        var value = img.getValueXY(_x, h, 0);
        if (value < min) {
          min = value;
        }
      }
      minList[_x] = min;
    }

    for (var _x2 = 0; _x2 < img.width; _x2++) {
      var _min = img.maxValue;
      for (var i = Math.max(0, _x2 - radiusX); i < Math.min(img.width, _x2 + radiusX + 1); i++) {
        if (minList[i] < _min) {
          _min = minList[i];
        }
      }
      newImage.setValueXY(_x2, y, 0, _min);
    }
  }
  return newImage;
}

function erodeOnceBinary(img, newImage, kernel) {
  var kernelWidth = kernel.length;
  var kernelHeight = kernel[0].length;
  var radiusX = (kernelWidth - 1) / 2;
  var radiusY = (kernelHeight - 1) / 2;
  for (var y = 0; y < img.height; y++) {
    for (var x = 0; x < img.width; x++) {
      var min = 1;
      intLoop: for (var jj = 0; jj < kernelHeight; jj++) {
        for (var ii = 0; ii < kernelWidth; ii++) {
          if (kernel[ii][jj] !== 1) continue;
          var i = ii - radiusX + x;
          var j = jj - radiusY + y;
          if (j < 0 || i < 0 || i >= img.width || j >= img.height) continue;
          var value = img.getBitXY(i, j);
          if (value === 0) {
            min = 0;
            break intLoop;
          }
        }
      }
      if (min === 1) {
        newImage.setBitXY(x, y);
      }
    }
  }
  return newImage;
}

function erodeOnceBinaryOnlyOnes(img, newImage, kernelWidth, kernelHeight) {
  var radiusX = (kernelWidth - 1) / 2;
  var radiusY = (kernelHeight - 1) / 2;

  var minList = [];
  for (var x = 0; x < img.width; x++) {
    minList.push(0);
  }

  for (var y = 0; y < img.height; y++) {
    for (var _x3 = 0; _x3 < img.width; _x3++) {
      minList[_x3] = 1;
      for (var h = Math.max(0, y - radiusY); h < Math.min(img.height, y + radiusY + 1); h++) {
        if (img.getBitXY(_x3, h) === 0) {
          minList[_x3] = 0;
          break;
        }
      }
    }

    for (var _x4 = 0; _x4 < img.width; _x4++) {
      if (newImage.getBitXY(_x4, y) === 0) continue;
      for (var i = Math.max(0, _x4 - radiusX); i < Math.min(img.width, _x4 + radiusX + 1); i++) {
        if (minList[i] === 0) {
          newImage.clearBitXY(_x4, y);
          break;
        }
      }
    }
  }
  return newImage;
}